@@ -3,28 +3,29 @@
  * joinrels.c
  *	  Routines to determine which relations should be joined
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/joinrels.c,v 1.63.4.2 2004/01/24 00:37:42 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/path/joinrels.c,v 1.76.2.1 2005/11/22 18:23:10 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
+#include "optimizer/joininfo.h"
 #include "optimizer/pathnode.h"
 #include "optimizer/paths.h"
 
 
-static List *make_rels_by_clause_joins(Query *root,
+static List *make_rels_by_clause_joins(PlannerInfo *root,
 						  RelOptInfo *old_rel,
-						  List *other_rels);
-static List *make_rels_by_clauseless_joins(Query *root,
+						  ListCell *other_rels);
+static List *make_rels_by_clauseless_joins(PlannerInfo *root,
 							  RelOptInfo *old_rel,
-							  List *other_rels);
-static bool is_inside_IN(Query *root, RelOptInfo *rel);
+							  ListCell *other_rels);
+static bool is_inside_IN(PlannerInfo *root, RelOptInfo *rel);
 
 
 /*
@@ -39,57 +40,57 @@ static bool is_inside_IN(Query *root, RelOptInfo *rel);
  * joinrels[j], 1 <= j < level, is a list of rels containing j items.
  */
 List *
-make_rels_by_joins(Query *root, int level, List **joinrels)
+make_rels_by_joins(PlannerInfo *root, int level, List **joinrels)
 {
 	List	   *result_rels = NIL;
 	List	   *new_rels;
-	List	   *nr;
-	List	   *r;
+	ListCell   *r;
 	int			k;
 
 	/*
 	 * First, consider left-sided and right-sided plans, in which rels of
-	 * exactly level-1 member relations are joined against initial
-	 * relations. We prefer to join using join clauses, but if we find a
-	 * rel of level-1 members that has no join clauses, we will generate
-	 * Cartesian-product joins against all initial rels not already
-	 * contained in it.
+	 * exactly level-1 member relations are joined against initial relations.
+	 * We prefer to join using join clauses, but if we find a rel of level-1
+	 * members that has no join clauses, we will generate Cartesian-product
+	 * joins against all initial rels not already contained in it.
 	 *
-	 * In the first pass (level == 2), we try to join each initial rel to
-	 * each initial rel that appears later in joinrels[1].	(The
-	 * mirror-image joins are handled automatically by make_join_rel.)	In
-	 * later passes, we try to join rels of size level-1 from
-	 * joinrels[level-1] to each initial rel in joinrels[1].
+	 * In the first pass (level == 2), we try to join each initial rel to each
+	 * initial rel that appears later in joinrels[1].  (The mirror-image joins
+	 * are handled automatically by make_join_rel.)  In later passes, we try
+	 * to join rels of size level-1 from joinrels[level-1] to each initial rel
+	 * in joinrels[1].
 	 */
 	foreach(r, joinrels[level - 1])
 	{
 		RelOptInfo *old_rel = (RelOptInfo *) lfirst(r);
-		List	   *other_rels;
+		ListCell   *other_rels;
 
 		if (level == 2)
 			other_rels = lnext(r);		/* only consider remaining initial
 										 * rels */
 		else
-			other_rels = joinrels[1];	/* consider all initial rels */
+			other_rels = list_head(joinrels[1]);		/* consider all initial
+														 * rels */
 
 		if (old_rel->joininfo != NIL)
 		{
 			/*
-			 * Note that if all available join clauses for this rel
-			 * require more than one other rel, we will fail to make any
-			 * joins against it here.  In most cases that's OK; it'll be
-			 * considered by "bushy plan" join code in a higher-level pass
-			 * where we have those other rels collected into a join rel.
+			 * Note that if all available join clauses for this rel require
+			 * more than one other rel, we will fail to make any joins against
+			 * it here.  In most cases that's OK; it'll be considered by
+			 * "bushy plan" join code in a higher-level pass where we have
+			 * those other rels collected into a join rel.
 			 */
 			new_rels = make_rels_by_clause_joins(root,
 												 old_rel,
 												 other_rels);
+
 			/*
 			 * An exception occurs when there is a clauseless join inside an
 			 * IN (sub-SELECT) construct.  Here, the members of the subselect
 			 * all have join clauses (against the stuff outside the IN), but
 			 * they *must* be joined to each other before we can make use of
-			 * those join clauses.  So do the clauseless join bit.
+			 * those join clauses.	So do the clauseless join bit.
 			 *
 			 * See also the last-ditch case below.
 			 */
@@ -112,36 +113,29 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 		/*
 		 * At levels above 2 we will generate the same joined relation in
 		 * multiple ways --- for example (a join b) join c is the same
-		 * RelOptInfo as (b join c) join a, though the second case will
-		 * add a different set of Paths to it.	To avoid making extra work
-		 * for subsequent passes, do not enter the same RelOptInfo into
-		 * our output list multiple times.
+		 * RelOptInfo as (b join c) join a, though the second case will add a
+		 * different set of Paths to it.  To avoid making extra work for
+		 * subsequent passes, do not enter the same RelOptInfo into our output
+		 * list multiple times.
 		 */
-		foreach(nr, new_rels)
-		{
-			RelOptInfo *jrel = (RelOptInfo *) lfirst(nr);
-
-			if (!ptrMember(jrel, result_rels))
-				result_rels = lcons(jrel, result_rels);
-		}
+		result_rels = list_concat_unique_ptr(result_rels, new_rels);
 	}
 
 	/*
-	 * Now, consider "bushy plans" in which relations of k initial rels
-	 * are joined to relations of level-k initial rels, for 2 <= k <=
-	 * level-2.
+	 * Now, consider "bushy plans" in which relations of k initial rels are
+	 * joined to relations of level-k initial rels, for 2 <= k <= level-2.
 	 *
 	 * We only consider bushy-plan joins for pairs of rels where there is a
-	 * suitable join clause, in order to avoid unreasonable growth of
-	 * planning time.
+	 * suitable join clause, in order to avoid unreasonable growth of planning
+	 * time.
 	 */
 	for (k = 2;; k++)
 	{
 		int			other_level = level - k;
 
 		/*
-		 * Since make_join_rel(x, y) handles both x,y and y,x cases, we
-		 * only need to go as far as the halfway point.
+		 * Since make_join_rel(x, y) handles both x,y and y,x cases, we only
+		 * need to go as far as the halfway point.
 		 */
 		if (k > other_level)
 			break;
@@ -149,8 +143,8 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 		foreach(r, joinrels[k])
 		{
 			RelOptInfo *old_rel = (RelOptInfo *) lfirst(r);
-			List	   *other_rels;
-			List	   *r2;
+			ListCell   *other_rels;
+			ListCell   *r2;
 
 			if (old_rel->joininfo == NIL)
 				continue;		/* we ignore clauseless joins here */
@@ -158,38 +152,29 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 			if (k == other_level)
 				other_rels = lnext(r);	/* only consider remaining rels */
 			else
-				other_rels = joinrels[other_level];
+				other_rels = list_head(joinrels[other_level]);
 
-			foreach(r2, other_rels)
+			for_each_cell(r2, other_rels)
 			{
 				RelOptInfo *new_rel = (RelOptInfo *) lfirst(r2);
 
 				if (!bms_overlap(old_rel->relids, new_rel->relids))
 				{
-					List	   *i;
-
 					/*
 					 * OK, we can build a rel of the right level from this
-					 * pair of rels.  Do so if there is at least one
-					 * usable join clause.
+					 * pair of rels.  Do so if there is at least one usable
+					 * join clause.
 					 */
-					foreach(i, old_rel->joininfo)
+					if (have_relevant_joinclause(old_rel, new_rel))
 					{
-						JoinInfo   *joininfo = (JoinInfo *) lfirst(i);
+						RelOptInfo *jrel;
 
-						if (bms_is_subset(joininfo->unjoined_relids,
-										  new_rel->relids))
-						{
-							RelOptInfo *jrel;
-
-							jrel = make_join_rel(root, old_rel, new_rel,
-												 JOIN_INNER);
-							/* Avoid making duplicate entries ... */
-							if (jrel && !ptrMember(jrel, result_rels))
-								result_rels = lcons(jrel, result_rels);
-							break;		/* need not consider more
-										 * joininfos */
-						}
+						jrel = make_join_rel(root, old_rel, new_rel,
+											 JOIN_INNER);
+						/* Avoid making duplicate entries ... */
+						if (jrel)
+							result_rels = list_append_unique_ptr(result_rels,
+																 jrel);
 					}
 				}
 			}
@@ -197,16 +182,16 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 	}
 
 	/*
-	 * Last-ditch effort: if we failed to find any usable joins so far,
-	 * force a set of cartesian-product joins to be generated.	This
-	 * handles the special case where all the available rels have join
-	 * clauses but we cannot use any of the joins yet.	An example is
+	 * Last-ditch effort: if we failed to find any usable joins so far, force
+	 * a set of cartesian-product joins to be generated.  This handles the
+	 * special case where all the available rels have join clauses but we
+	 * cannot use any of the joins yet.  An example is
 	 *
 	 * SELECT * FROM a,b,c WHERE (a.f1 + b.f2 + c.f3) = 0;
 	 *
 	 * The join clause will be usable at level 3, but at level 2 we have no
-	 * choice but to make cartesian joins.	We consider only left-sided
-	 * and right-sided cartesian joins in this case (no bushy).
+	 * choice but to make cartesian joins.	We consider only left-sided and
+	 * right-sided cartesian joins in this case (no bushy).
 	 */
 	if (result_rels == NIL)
 	{
@@ -217,35 +202,29 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 		foreach(r, joinrels[level - 1])
 		{
 			RelOptInfo *old_rel = (RelOptInfo *) lfirst(r);
-			List	   *other_rels;
+			ListCell   *other_rels;
 
 			if (level == 2)
 				other_rels = lnext(r);	/* only consider remaining initial
 										 * rels */
 			else
-				other_rels = joinrels[1];		/* consider all initial
-												 * rels */
+				other_rels = list_head(joinrels[1]);	/* consider all initial
+														 * rels */
 
 			new_rels = make_rels_by_clauseless_joins(root,
 													 old_rel,
 													 other_rels);
 
-			foreach(nr, new_rels)
-			{
-				RelOptInfo *jrel = (RelOptInfo *) lfirst(nr);
-
-				if (!ptrMember(jrel, result_rels))
-					result_rels = lcons(jrel, result_rels);
-			}
+			result_rels = list_concat_unique_ptr(result_rels, new_rels);
 		}
 
 		/*----------
 		 * When IN clauses are involved, there may be no legal way to make
-		 * an N-way join for some values of N.  For example consider
+		 * an N-way join for some values of N.	For example consider
 		 *
 		 * SELECT ... FROM t1 WHERE
-		 *   x IN (SELECT ... FROM t2,t3 WHERE ...) AND
-		 *   y IN (SELECT ... FROM t4,t5 WHERE ...)
+		 *	 x IN (SELECT ... FROM t2,t3 WHERE ...) AND
+		 *	 y IN (SELECT ... FROM t4,t5 WHERE ...)
 		 *
 		 * We will flatten this query to a 5-way join problem, but there are
 		 * no 4-way joins that make_join_rel() will consider legal.  We have
@@ -266,50 +245,37 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 /*
  * make_rels_by_clause_joins
  *	  Build joins between the given relation 'old_rel' and other relations
- *	  that are mentioned within old_rel's joininfo nodes (i.e., relations
+ *	  that are mentioned within old_rel's joininfo list (i.e., relations
  *	  that participate in join clauses that 'old_rel' also participates in).
  *	  The join rel nodes are returned in a list.
  *
  * 'old_rel' is the relation entry for the relation to be joined
- * 'other_rels': other rels to be considered for joining
+ * 'other_rels': the first cell in a linked list containing the other
+ * rels to be considered for joining
  *
  * Currently, this is only used with initial rels in other_rels, but it
- * will work for joining to joinrels too, if the caller ensures there is no
- * membership overlap between old_rel and the rels in other_rels.  (We need
- * no extra test for overlap for initial rels, since the is_subset test can
- * only succeed when other_rel is not already part of old_rel.)
+ * will work for joining to joinrels too.
  */
 static List *
-make_rels_by_clause_joins(Query *root,
+make_rels_by_clause_joins(PlannerInfo *root,
 						  RelOptInfo *old_rel,
-						  List *other_rels)
+						  ListCell *other_rels)
 {
 	List	   *result = NIL;
-	List	   *i,
-			   *j;
+	ListCell   *l;
 
-	foreach(i, old_rel->joininfo)
+	for_each_cell(l, other_rels)
 	{
-		JoinInfo   *joininfo = (JoinInfo *) lfirst(i);
-		Relids		unjoined_relids = joininfo->unjoined_relids;
+		RelOptInfo *other_rel = (RelOptInfo *) lfirst(l);
 
-		foreach(j, other_rels)
+		if (!bms_overlap(old_rel->relids, other_rel->relids) &&
+			have_relevant_joinclause(old_rel, other_rel))
 		{
-			RelOptInfo *other_rel = (RelOptInfo *) lfirst(j);
+			RelOptInfo *jrel;
 
-			if (bms_is_subset(unjoined_relids, other_rel->relids))
-			{
-				RelOptInfo *jrel;
-
-				jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
-
-				/*
-				 * Avoid entering same joinrel into our output list more
-				 * than once.
-				 */
-				if (jrel && !ptrMember(jrel, result))
-					result = lcons(jrel, result);
-			}
+			jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
+			if (jrel)
+				result = lcons(jrel, result);
 		}
 	}
 
@@ -324,20 +290,21 @@ make_rels_by_clause_joins(Query *root,
  *	  The join rel nodes are returned in a list.
  *
  * 'old_rel' is the relation entry for the relation to be joined
- * 'other_rels': other rels to be considered for joining
+ * 'other_rels': the first cell of a linked list containing the
+ * other rels to be considered for joining
  *
  * Currently, this is only used with initial rels in other_rels, but it would
  * work for joining to joinrels too.
  */
 static List *
-make_rels_by_clauseless_joins(Query *root,
+make_rels_by_clauseless_joins(PlannerInfo *root,
 							  RelOptInfo *old_rel,
-							  List *other_rels)
+							  ListCell *other_rels)
 {
 	List	   *result = NIL;
-	List	   *i;
+	ListCell   *i;
 
-	foreach(i, other_rels)
+	for_each_cell(i, other_rels)
 	{
 		RelOptInfo *other_rel = (RelOptInfo *) lfirst(i);
 
@@ -348,8 +315,8 @@ make_rels_by_clauseless_joins(Query *root,
 			jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
 
 			/*
-			 * As long as given other_rels are distinct, don't need to
-			 * test to see if jrel is already part of output list.
+			 * As long as given other_rels are distinct, don't need to test to
+			 * see if jrel is already part of output list.
 			 */
 			if (jrel)
 				result = lcons(jrel, result);
@@ -368,13 +335,13 @@ make_rels_by_clauseless_joins(Query *root,
  * out of an IN, so the routine name is a slight misnomer.
  */
 static bool
-is_inside_IN(Query *root, RelOptInfo *rel)
+is_inside_IN(PlannerInfo *root, RelOptInfo *rel)
 {
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, root->in_info_list)
+	foreach(l, root->in_info_list)
 	{
-		InClauseInfo *ininfo = (InClauseInfo *) lfirst(i);
+		InClauseInfo *ininfo = (InClauseInfo *) lfirst(l);
 
 		if (bms_is_subset(rel->relids, ininfo->righthand))
 			return true;
@@ -390,7 +357,7 @@ is_inside_IN(Query *root, RelOptInfo *rel)
  *		path that corresponds exactly to what the user wrote.
  */
 RelOptInfo *
-make_jointree_rel(Query *root, Node *jtnode)
+make_jointree_rel(PlannerInfo *root, Node *jtnode)
 {
 	if (IsA(jtnode, RangeTblRef))
 	{
@@ -423,10 +390,10 @@ make_jointree_rel(Query *root, Node *jtnode)
 			elog(ERROR, "invalid join order");
 
 		/*
-		 * Since we are only going to consider this one way to do it,
-		 * we're done generating Paths for this joinrel and can now select
-		 * the cheapest.  In fact we *must* do so now, since next level up
-		 * will need it!
+		 * Since we are only going to consider this one way to do it, we're
+		 * done generating Paths for this joinrel and can now select the
+		 * cheapest.  In fact we *must* do so now, since next level up will
+		 * need it!
 		 */
 		set_cheapest(rel);
 
@@ -455,7 +422,7 @@ make_jointree_rel(Query *root, Node *jtnode)
  * happen when working with IN clauses that have been turned into joins.
  */
 RelOptInfo *
-make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
+make_join_rel(PlannerInfo *root, RelOptInfo *rel1, RelOptInfo *rel2,
 			  JoinType jointype)
 {
 	Relids		joinrelids;
@@ -469,49 +436,59 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 	joinrelids = bms_union(rel1->relids, rel2->relids);
 
 	/*
-	 * If we are implementing IN clauses as joins, there are some joins
-	 * that are illegal.  Check to see if the proposed join is trouble. We
-	 * can skip the work if looking at an outer join, however, because
-	 * only top-level joins might be affected.
+	 * If we are implementing IN clauses as joins, there are some joins that
+	 * are illegal.  Check to see if the proposed join is trouble. We can skip
+	 * the work if looking at an outer join, however, because only top-level
+	 * joins might be affected.
 	 */
 	if (jointype == JOIN_INNER)
 	{
-		List	   *l;
+		ListCell   *l;
 
 		foreach(l, root->in_info_list)
 		{
 			InClauseInfo *ininfo = (InClauseInfo *) lfirst(l);
 
 			/*
-			 * Cannot join if proposed join contains part, but only part,
-			 * of the RHS, *and* it contains rels not in the RHS.
+			 * This IN clause is not relevant unless its RHS overlaps the
+			 * proposed join.  (Check this first as a fast path for dismissing
+			 * most irrelevant INs quickly.)
 			 */
-			if (bms_overlap(ininfo->righthand, joinrelids) &&
-				!bms_is_subset(ininfo->righthand, joinrelids) &&
-				!bms_is_subset(joinrelids, ininfo->righthand))
+			if (!bms_overlap(ininfo->righthand, joinrelids))
+				continue;
+
+			/*
+			 * If we are still building the IN clause's RHS, then this IN
+			 * clause isn't relevant yet.
+			 */
+			if (bms_is_subset(joinrelids, ininfo->righthand))
+				continue;
+
+			/*
+			 * Cannot join if proposed join contains rels not in the RHS *and*
+			 * contains only part of the RHS.  We must build the complete RHS
+			 * (subselect's join) before it can be joined to rels outside the
+			 * subselect.
+			 */
+			if (!bms_is_subset(ininfo->righthand, joinrelids))
 			{
 				bms_free(joinrelids);
 				return NULL;
 			}
 
 			/*
-			 * No issue unless we are looking at a join of the IN's RHS to
-			 * other stuff.
+			 * At this point we are considering a join of the IN's RHS to some
+			 * other rel(s).
+			 *
+			 * If we already joined IN's RHS to any other rels in either input
+			 * path, then this join is not constrained (the necessary work was
+			 * done at the lower level where that join occurred).
 			 */
-			if (!(bms_is_subset(ininfo->righthand, joinrelids) &&
-				  !bms_equal(ininfo->righthand, joinrelids)))
+			if (bms_is_subset(ininfo->righthand, rel1->relids) &&
+				!bms_equal(ininfo->righthand, rel1->relids))
 				continue;
-
-			/*
-			 * If we already joined IN's RHS to any part of its LHS in
-			 * either input path, then this join is not constrained (the
-			 * necessary work was done at a lower level).
-			 */
-			if (bms_overlap(ininfo->lefthand, rel1->relids) &&
-				bms_is_subset(ininfo->righthand, rel1->relids))
-				continue;
-			if (bms_overlap(ininfo->lefthand, rel2->relids) &&
-				bms_is_subset(ininfo->righthand, rel2->relids))
+			if (bms_is_subset(ininfo->righthand, rel2->relids) &&
+				!bms_equal(ininfo->righthand, rel2->relids))
 				continue;
 
 			/*
@@ -520,11 +497,11 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 			 * RHS/LHS.
 			 *
 			 * JOIN_UNIQUE_OUTER will work if outerrel is exactly RHS;
-			 * conversely JOIN_UNIQUE_INNER will work if innerrel is
-			 * exactly RHS.
+			 * conversely JOIN_UNIQUE_INNER will work if innerrel is exactly
+			 * RHS.
 			 *
-			 * But none of these will work if we already found another IN
-			 * that needs to trigger here.
+			 * But none of these will work if we already found another IN that
+			 * needs to trigger here.
 			 */
 			if (jointype != JOIN_INNER)
 			{
@@ -551,8 +528,8 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 	}
 
 	/*
-	 * Find or build the join RelOptInfo, and compute the restrictlist
-	 * that goes with this particular joining.
+	 * Find or build the join RelOptInfo, and compute the restrictlist that
+	 * goes with this particular joining.
 	 */
 	joinrel = build_join_rel(root, joinrelids, rel1, rel2, jointype,
 							 &restrictlist);
