@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000-2005, PostgreSQL Global Development Group
  *
- * $PostgreSQL: pgsql/src/bin/psql/common.c,v 1.110 2005/11/04 18:35:40 tgl Exp $
+ * $PostgreSQL: pgsql/src/bin/psql/common.c,v 1.110.2.3 2006/06/30 16:55:41 alvherre Exp $
  */
 #include "postgres_fe.h"
 #include "common.h"
@@ -268,7 +268,6 @@ handle_sigint(SIGNAL_ARGS)
 	}
 	errno = save_errno;			/* just in case the write changed it */
 }
-
 #else							/* WIN32 */
 
 static BOOL WINAPI
@@ -323,7 +322,6 @@ setup_cancel_handler(void)
 		done = true;
 	}
 }
-
 #endif   /* WIN32 */
 
 
@@ -531,9 +529,9 @@ ReportSyntaxErrorPosition(const PGresult *result, const char *query)
 		 * want to think about coping with their variable screen width, but
 		 * not today.)
 		 *
-		 * Extract line number and begin and end indexes of line containing error
-		 * location.  There will not be any newlines or carriage returns in
-		 * the selected extract.
+		 * Extract line number and begin and end indexes of line containing
+		 * error location.	There will not be any newlines or carriage returns
+		 * in the selected extract.
 		 */
 		for (i = 0; i < clen; i++)
 		{
@@ -1069,19 +1067,19 @@ SendQuery(const char *query)
 	if (OK)
 		OK = PrintQueryResults(results);
 
-	PQclear(results);
-
 	/* If we made a temporary savepoint, possibly release/rollback */
 	if (on_error_rollback_savepoint)
 	{
+		PGresult   *svptres;
+
 		transaction_status = PQtransactionStatus(pset.db);
 
 		/* We always rollback on an error */
 		if (transaction_status == PQTRANS_INERROR)
-			results = PQexec(pset.db, "ROLLBACK TO pg_psql_temporary_savepoint");
+			svptres = PQexec(pset.db, "ROLLBACK TO pg_psql_temporary_savepoint");
 		/* If they are no longer in a transaction, then do nothing */
 		else if (transaction_status != PQTRANS_INTRANS)
-			results = NULL;
+			svptres = NULL;
 		else
 		{
 			/*
@@ -1092,20 +1090,24 @@ SendQuery(const char *query)
 			if (strcmp(PQcmdStatus(results), "SAVEPOINT") == 0 ||
 				strcmp(PQcmdStatus(results), "RELEASE") == 0 ||
 				strcmp(PQcmdStatus(results), "ROLLBACK") == 0)
-				results = NULL;
+				svptres = NULL;
 			else
-				results = PQexec(pset.db, "RELEASE pg_psql_temporary_savepoint");
+				svptres = PQexec(pset.db, "RELEASE pg_psql_temporary_savepoint");
 		}
-		if (PQresultStatus(results) != PGRES_COMMAND_OK)
+		if (svptres && PQresultStatus(svptres) != PGRES_COMMAND_OK)
 		{
 			psql_error("%s", PQerrorMessage(pset.db));
 			PQclear(results);
+			PQclear(svptres);
 			ResetCancelConn();
 			return false;
 		}
-		PQclear(results);
+
+		PQclear(svptres);
 	}
 
+	PQclear(results);
+
 	/* Possible microtiming output */
 	if (OK && pset.timing)
 		printf(_("Time: %.3f ms\n"), DIFF_MSEC(&after, &before));
@@ -1217,8 +1219,8 @@ command_no_begin(const char *query)
 	 * gives rise to a TransactionStmt in the backend grammar, except for the
 	 * savepoint-related commands.
 	 *
-	 * (We assume that START must be START TRANSACTION, since there is presently
-	 * no other "START foo" command.)
+	 * (We assume that START must be START TRANSACTION, since there is
+	 * presently no other "START foo" command.)
 	 */
 	if (wordlen == 5 && pg_strncasecmp(query, "abort", 5) == 0)
 		return true;
